@@ -49,13 +49,13 @@ using namespace std;
 
 CCECProcessor::CCECProcessor(CLibCEC *controller, const char *strDeviceName, cec_logical_address iLogicalAddress /* = CECDEVICE_PLAYBACKDEVICE1 */, uint16_t iPhysicalAddress /* = CEC_DEFAULT_PHYSICAL_ADDRESS*/) :
     m_bStarted(false),
+    m_bInitialised(false),
     m_iHDMIPort(CEC_DEFAULT_HDMI_PORT),
     m_iBaseDevice((cec_logical_address)CEC_DEFAULT_BASE_DEVICE),
     m_lastInitiator(CECDEVICE_UNKNOWN),
     m_strDeviceName(strDeviceName),
     m_controller(controller),
     m_bMonitor(false),
-    m_busScan(NULL),
     m_iStandardLineTimeout(3),
     m_iRetryLineTimeout(3),
     m_iLastTransmission(0)
@@ -70,6 +70,7 @@ CCECProcessor::CCECProcessor(CLibCEC *controller, const char *strDeviceName, cec
 
 CCECProcessor::CCECProcessor(CLibCEC *controller, const char *strDeviceName, const cec_device_type_list &types) :
     m_bStarted(false),
+    m_bInitialised(false),
     m_iHDMIPort(CEC_DEFAULT_HDMI_PORT),
     m_iBaseDevice((cec_logical_address)CEC_DEFAULT_BASE_DEVICE),
     m_strDeviceName(strDeviceName),
@@ -130,46 +131,63 @@ CCECProcessor::~CCECProcessor(void)
 
 bool CCECProcessor::Start(const char *strPort, uint16_t iBaudRate /* = 38400 */, uint32_t iTimeoutMs /* = 10000 */)
 {
-  CLockObject lock(&m_mutex);
-  if (!m_communication || m_communication->IsOpen())
-  {
-    m_controller->AddLog(CEC_LOG_ERROR, "connection already opened");
-    return false;
-  }
+  bool bReturn(false);
 
-  if (!m_communication->Open(strPort, iBaudRate, iTimeoutMs))
   {
-    m_controller->AddLog(CEC_LOG_ERROR, "could not open a connection");
-    return false;
-  }
+    CLockObject lock(&m_mutex);
 
-  if (CreateThread())
-  {
-    if (!m_startCondition.Wait(&m_mutex) || !m_bStarted)
+    /* check for an already opened connection */
+    if (!m_communication || m_communication->IsOpen())
+    {
+      m_controller->AddLog(CEC_LOG_ERROR, "connection already opened");
+      return bReturn;
+    }
+
+    /* open a new connection */
+    if (!m_communication->Open(strPort, iBaudRate, iTimeoutMs))
+    {
+      m_controller->AddLog(CEC_LOG_ERROR, "could not open a connection");
+      return bReturn;
+    }
+
+    /* create the processor thread */
+    if (!CreateThread() || !m_startCondition.Wait(&m_mutex) || !m_bStarted)
     {
       m_controller->AddLog(CEC_LOG_ERROR, "could not create a processor thread");
-      return false;
-    }
-
-    lock.Leave();
-
-    m_busDevices[CECDEVICE_TV]->GetVendorId();
-
-    if (SetHDMIPort(m_iBaseDevice, m_iHDMIPort, true))
-    {
-      m_controller->AddLog(CEC_LOG_DEBUG, "processor thread started");
-      m_busScan = new CCECBusScan(this);
-      m_busScan->CreateThread(true);
-      return true;
-    }
-    else
-    {
-      m_controller->AddLog(CEC_LOG_ERROR, "failed to initialise the processor");
+      return bReturn;
     }
   }
 
-  m_controller->AddLog(CEC_LOG_ERROR, "could not create a processor thread");
-  return false;
+  /* find the logical address for the adapter */
+  bReturn = m_logicalAddresses.IsEmpty() ? FindLogicalAddresses() : true;
+  if (!bReturn)
+    m_controller->AddLog(CEC_LOG_ERROR, "could not detect our logical addresses");
+
+  /* set the physical address for the adapter */
+  if (bReturn)
+  {
+    /* only set our OSD name for the primary device */
+    m_busDevices[m_logicalAddresses.primary]->m_strDeviceName = m_strDeviceName;
+
+    /* get the vendor id from the TV, so we are using the correct handler */
+    m_busDevices[CECDEVICE_TV]->RequestVendorId();
+    ReplaceHandlers();
+
+    bReturn = SetHDMIPort(m_iBaseDevice, m_iHDMIPort, true);
+  }
+
+  if (bReturn)
+  {
+    m_bInitialised = true;
+    m_controller->AddLog(CEC_LOG_DEBUG, "processor thread started");
+  }
+  else
+  {
+    m_controller->AddLog(CEC_LOG_ERROR, "could not create a processor thread");
+    StopThread(true);
+  }
+
+  return bReturn;
 }
 
 bool CCECProcessor::TryLogicalAddress(cec_logical_address address)
@@ -214,6 +232,85 @@ bool CCECProcessor::FindLogicalAddressAudioSystem(void)
   return TryLogicalAddress(CECDEVICE_AUDIOSYSTEM);
 }
 
+bool CCECProcessor::ChangeDeviceType(cec_device_type from, cec_device_type to)
+{
+  bool bChanged(false);
+
+  CStdString strLog;
+  strLog.Format("changing device type '%s' into '%s'", ToString(from), ToString(to));
+  AddLog(CEC_LOG_NOTICE, strLog);
+
+  CLockObject lock(&m_mutex);
+  CCECBusDevice *previousDevice = GetDeviceByType(from);
+  m_logicalAddresses.primary = CECDEVICE_UNKNOWN;
+
+  for (unsigned int iPtr = 0; iPtr < 5; iPtr++)
+  {
+    if (m_types.types[iPtr] == CEC_DEVICE_TYPE_RESERVED)
+      continue;
+
+    if (m_types.types[iPtr] == from)
+    {
+      bChanged = true;
+      m_types.types[iPtr] = to;
+    }
+    else if (m_types.types[iPtr] == to && bChanged)
+    {
+      m_types.types[iPtr] = CEC_DEVICE_TYPE_RESERVED;
+    }
+  }
+
+  if (bChanged)
+  {
+    FindLogicalAddresses();
+
+    CCECBusDevice *newDevice = GetDeviceByType(to);
+    if (previousDevice && newDevice)
+    {
+      newDevice->SetDeviceStatus(CEC_DEVICE_STATUS_HANDLED_BY_LIBCEC);
+      previousDevice->SetDeviceStatus(CEC_DEVICE_STATUS_UNKNOWN);
+
+      newDevice->SetCecVersion(previousDevice->GetCecVersion(false));
+      previousDevice->SetCecVersion(CEC_VERSION_UNKNOWN);
+
+      newDevice->SetMenuLanguage(previousDevice->GetMenuLanguage(false));
+      cec_menu_language lang;
+      lang.device = previousDevice->GetLogicalAddress();
+      for (unsigned int iPtr = 0; iPtr < 4; iPtr++)
+        lang.language[iPtr] = '?';
+      lang.language[3] = 0;
+      previousDevice->SetMenuLanguage(lang);
+
+      newDevice->SetMenuState(previousDevice->GetMenuState());
+      previousDevice->SetMenuState(CEC_MENU_STATE_DEACTIVATED);
+
+      newDevice->SetOSDName(previousDevice->GetOSDName(false));
+      previousDevice->SetOSDName(ToString(previousDevice->GetLogicalAddress()));
+
+      newDevice->SetPhysicalAddress(previousDevice->GetPhysicalAddress(false));
+      previousDevice->SetPhysicalAddress(0xFFFF);
+
+      newDevice->SetPowerStatus(previousDevice->GetPowerStatus(false));
+      previousDevice->SetPowerStatus(CEC_POWER_STATUS_UNKNOWN);
+
+      newDevice->SetVendorId(previousDevice->GetVendorId(false));
+      previousDevice->SetVendorId(CEC_VENDOR_UNKNOWN);
+
+      if ((from == CEC_DEVICE_TYPE_PLAYBACK_DEVICE || from == CEC_DEVICE_TYPE_RECORDING_DEVICE) &&
+          (to == CEC_DEVICE_TYPE_PLAYBACK_DEVICE || to == CEC_DEVICE_TYPE_RECORDING_DEVICE))
+      {
+        ((CCECPlaybackDevice *) newDevice)->SetDeckControlMode(((CCECPlaybackDevice *) previousDevice)->GetDeckControlMode());
+        ((CCECPlaybackDevice *) previousDevice)->SetDeckControlMode(CEC_DECK_CONTROL_MODE_STOP);
+
+        ((CCECPlaybackDevice *) newDevice)->SetDeckStatus(((CCECPlaybackDevice *) previousDevice)->GetDeckStatus());
+        ((CCECPlaybackDevice *) previousDevice)->SetDeckStatus(CEC_DECK_INFO_STOP);
+      }
+    }
+  }
+
+  return true;
+}
+
 bool CCECProcessor::FindLogicalAddresses(void)
 {
   bool bReturn(true);
@@ -238,30 +335,25 @@ bool CCECProcessor::FindLogicalAddresses(void)
       bReturn &= FindLogicalAddressAudioSystem();
   }
 
+  if (bReturn)
+    SetAckMask(m_logicalAddresses.AckMask());
+
   return bReturn;
 }
 
+void CCECProcessor::ReplaceHandlers(void)
+{
+  for (uint8_t iPtr = 0; iPtr <= CECDEVICE_PLAYBACKDEVICE3; iPtr++)
+    m_busDevices[iPtr]->ReplaceHandler(m_bInitialised);
+}
+
 void *CCECProcessor::Process(void)
 {
   bool                  bParseFrame(false);
   cec_command           command;
   CCECAdapterMessage    msg;
 
-  if (m_logicalAddresses.IsEmpty() && !FindLogicalAddresses())
   {
-    CLockObject lock(&m_mutex);
-    m_controller->AddLog(CEC_LOG_ERROR, "could not detect our logical addresses");
-    m_startCondition.Signal();
-    return NULL;
-  }
-  else
-  {
-    /* only set our OSD name and active source for the primary device */
-    m_busDevices[m_logicalAddresses.primary]->m_strDeviceName = m_strDeviceName;
-    m_busDevices[m_logicalAddresses.primary]->m_bActiveSource = true;
-
-    SetAckMask(m_logicalAddresses.AckMask());
-
     CLockObject lock(&m_mutex);
     m_bStarted = true;
     m_controller->AddLog(CEC_LOG_DEBUG, "processor thread started");
@@ -270,6 +362,7 @@ void *CCECProcessor::Process(void)
 
   while (!IsStopped())
   {
+    ReplaceHandlers();
     command.Clear();
     msg.clear();
 
@@ -295,13 +388,6 @@ void *CCECProcessor::Process(void)
     m_controller->CheckKeypressTimeout();
   }
 
-  if (m_busScan)
-  {
-    m_busScan->StopThread();
-    delete m_busScan;
-    m_busScan = NULL;
-  }
-
   if (m_communication)
     m_communication->Close();
 
@@ -330,12 +416,16 @@ bool CCECProcessor::SetActiveSource(cec_device_type type /* = CEC_DEVICE_TYPE_RE
   }
 
   m_busDevices[addr]->SetActiveSource();
-  bReturn = m_busDevices[addr]->TransmitActiveSource();
-
-  if (bReturn && (m_busDevices[addr]->GetType() == CEC_DEVICE_TYPE_PLAYBACK_DEVICE ||
-      m_busDevices[addr]->GetType() == CEC_DEVICE_TYPE_RECORDING_DEVICE))
+  if (m_busDevices[addr]->GetPhysicalAddress(false) != 0xFFFF)
   {
-    bReturn = ((CCECPlaybackDevice *)m_busDevices[addr])->TransmitDeckStatus(CECDEVICE_TV);
+    bReturn = m_busDevices[addr]->TransmitActiveSource();
+
+    if (bReturn && (m_busDevices[addr]->GetType() == CEC_DEVICE_TYPE_PLAYBACK_DEVICE ||
+        m_busDevices[addr]->GetType() == CEC_DEVICE_TYPE_RECORDING_DEVICE) &&
+        m_busDevices[addr]->GetHandler()->SendDeckStatusUpdateOnActiveSource())
+    {
+      bReturn = ((CCECPlaybackDevice *)m_busDevices[addr])->TransmitDeckStatus(CECDEVICE_TV);
+    }
   }
 
   return bReturn;
@@ -407,6 +497,7 @@ bool CCECProcessor::SetDeckInfo(cec_deck_info info, bool bSendUpdate /* = true *
 bool CCECProcessor::SetHDMIPort(cec_logical_address iBaseDevice, uint8_t iPort, bool bForce /* = false */)
 {
   bool bReturn(false);
+  CLockObject lock(&m_mutex);
 
   m_iBaseDevice = iBaseDevice;
   m_iHDMIPort = iPort;
@@ -419,46 +510,30 @@ bool CCECProcessor::SetHDMIPort(cec_logical_address iBaseDevice, uint8_t iPort,
 
   uint16_t iPhysicalAddress(0);
   if (iBaseDevice > CECDEVICE_TV)
+  {
+    lock.Leave();
     iPhysicalAddress = m_busDevices[iBaseDevice]->GetPhysicalAddress();
-
-  if (iPhysicalAddress == 0xffff)
-  {
-    SetPhysicalAddress((uint16_t)iPort * 0x1000);
-    bReturn = false;
+    lock.Lock();
   }
-  else
+
+  if (iPhysicalAddress < 0xffff)
   {
-    uint16_t iPos = 0;
     if (iPhysicalAddress == 0)
-      iPos = 0x1000;
+      iPhysicalAddress += 0x1000 * iPort;
     else if (iPhysicalAddress % 0x1000 == 0)
-      iPos = 0x100;
+      iPhysicalAddress += 0x100 * iPort;
     else if (iPhysicalAddress % 0x100 == 0)
-      iPos = 0x10;
+      iPhysicalAddress += 0x10 * iPort;
     else if (iPhysicalAddress % 0x10 == 0)
-      iPos = 0x1;
+      iPhysicalAddress += iPort;
 
-    while(!bReturn && iPos > 0)
-    {
-      iPhysicalAddress += (uint16_t)(iPort * iPos);
-      strLog.Format("checking physical address %4x", iPhysicalAddress);
-      AddLog(CEC_LOG_DEBUG, strLog);
-      if (PhysicalAddressInUse(iPhysicalAddress))
-      {
-        strLog.Format("physical address %4x is in use", iPhysicalAddress);
-        AddLog(CEC_LOG_DEBUG, strLog);
-        iPos = (iPos == 1) ? 0 : iPos / 0x10;
-      }
-      else
-      {
-        strLog.Format("physical address %4x is free", iPhysicalAddress);
-        AddLog(CEC_LOG_DEBUG, strLog);
-        SetPhysicalAddress(iPhysicalAddress);
-        bReturn = true;
-      }
-    }
+    SetPhysicalAddress(iPhysicalAddress);
+    bReturn = true;
   }
 
+  if (!bReturn)
+    m_controller->AddLog(CEC_LOG_ERROR, "failed to set the physical address");
+
   return bReturn;
 }
 
@@ -496,6 +571,7 @@ void CCECProcessor::LogOutput(const cec_command &data)
 
 bool CCECProcessor::SetLogicalAddress(cec_logical_address iLogicalAddress)
 {
+  CLockObject lock(&m_mutex);
   if (m_logicalAddresses.primary != iLogicalAddress)
   {
     CStdString strLog;
@@ -523,18 +599,23 @@ bool CCECProcessor::SetMenuState(cec_menu_state state, bool bSendUpdate /* = tru
   return true;
 }
 
-bool CCECProcessor::SetPhysicalAddress(uint16_t iPhysicalAddress)
+bool CCECProcessor::SetPhysicalAddress(uint16_t iPhysicalAddress, bool bSendUpdate /* = true */)
 {
+  bool bWasActiveSource(false);
+  CLockObject lock(&m_mutex);
   if (!m_logicalAddresses.IsEmpty())
   {
     for (uint8_t iPtr = 0; iPtr < 15; iPtr++)
       if (m_logicalAddresses[iPtr])
       {
+        bWasActiveSource |= m_busDevices[iPtr]->IsActiveSource();
         m_busDevices[iPtr]->SetInactiveSource();
         m_busDevices[iPtr]->SetPhysicalAddress(iPhysicalAddress);
-        m_busDevices[iPtr]->TransmitPhysicalAddress();
+        if (bSendUpdate)
+          m_busDevices[iPtr]->TransmitPhysicalAddress();
       }
-    return SetActiveView();
+
+    return bWasActiveSource && bSendUpdate ? SetActiveView() : true;
   }
   return false;
 }
@@ -548,24 +629,6 @@ bool CCECProcessor::SwitchMonitoring(bool bEnable)
   {
     CLockObject lock(&m_mutex);
     m_bMonitor = bEnable;
-
-    if (!bEnable)
-    {
-      if (!m_busScan)
-      {
-        m_busScan = new CCECBusScan(this);
-        m_busScan->CreateThread(true);
-      }
-    }
-    else
-    {
-      if (m_busScan)
-      {
-        m_busScan->StopThread();
-        delete m_busScan;
-        m_busScan = NULL;
-      }
-    }
   }
 
   if (bEnable)
@@ -585,29 +648,29 @@ bool CCECProcessor::PollDevice(cec_logical_address iAddress)
   return false;
 }
 
-uint8_t CCECProcessor::VolumeUp(void)
+uint8_t CCECProcessor::VolumeUp(bool bSendRelease /* = true */)
 {
   uint8_t status = 0;
   if (IsPresentDevice(CECDEVICE_AUDIOSYSTEM))
-    status = ((CCECAudioSystem *)m_busDevices[CECDEVICE_AUDIOSYSTEM])->VolumeUp();
+    status = ((CCECAudioSystem *)m_busDevices[CECDEVICE_AUDIOSYSTEM])->VolumeUp(bSendRelease);
 
   return status;
 }
 
-uint8_t CCECProcessor::VolumeDown(void)
+uint8_t CCECProcessor::VolumeDown(bool bSendRelease /* = true */)
 {
   uint8_t status = 0;
   if (IsPresentDevice(CECDEVICE_AUDIOSYSTEM))
-    status = ((CCECAudioSystem *)m_busDevices[CECDEVICE_AUDIOSYSTEM])->VolumeDown();
+    status = ((CCECAudioSystem *)m_busDevices[CECDEVICE_AUDIOSYSTEM])->VolumeDown(bSendRelease);
 
   return status;
 }
 
-uint8_t CCECProcessor::MuteAudio(void)
+uint8_t CCECProcessor::MuteAudio(bool bSendRelease /* = true */)
 {
   uint8_t status = 0;
   if (IsPresentDevice(CECDEVICE_AUDIOSYSTEM))
-    status = ((CCECAudioSystem *)m_busDevices[CECDEVICE_AUDIOSYSTEM])->MuteAudio();
+    status = ((CCECAudioSystem *)m_busDevices[CECDEVICE_AUDIOSYSTEM])->MuteAudio(bSendRelease);
 
   return status;
 }
@@ -634,9 +697,9 @@ CCECBusDevice *CCECProcessor::GetDeviceByType(cec_device_type type) const
 {
   CCECBusDevice *device = NULL;
 
-  for (unsigned int iPtr = 0; iPtr < 16; iPtr++)
+  for (uint8_t iPtr = 0; iPtr < 16; iPtr++)
   {
-    if (m_busDevices[iPtr]->m_type == type)
+    if (m_busDevices[iPtr]->m_type == type && m_logicalAddresses[iPtr])
     {
       device = m_busDevices[iPtr];
       break;
@@ -646,6 +709,15 @@ CCECBusDevice *CCECProcessor::GetDeviceByType(cec_device_type type) const
   return device;
 }
 
+CCECBusDevice *CCECProcessor::GetPrimaryDevice(void) const
+{
+  CCECBusDevice *device(NULL);
+  cec_logical_address primary = m_logicalAddresses.primary;
+  if (primary != CECDEVICE_UNKNOWN)
+    device = m_busDevices[primary];
+  return device;
+}
+
 cec_version CCECProcessor::GetDeviceCecVersion(cec_logical_address iAddress)
 {
   return m_busDevices[iAddress]->GetCecVersion();
@@ -1002,14 +1074,35 @@ bool CCECProcessor::SetAckMask(uint16_t iMask)
   return bReturn;
 }
 
-bool CCECProcessor::TransmitKeypress(cec_logical_address iDestination, cec_user_control_code key)
+bool CCECProcessor::TransmitKeypress(cec_logical_address iDestination, cec_user_control_code key, bool bWait /* = true */)
 {
-  return m_busDevices[iDestination]->TransmitKeypress(key);
+  return m_busDevices[iDestination]->TransmitKeypress(key, bWait);
 }
 
-bool CCECProcessor::TransmitKeyRelease(cec_logical_address iDestination)
+bool CCECProcessor::TransmitKeyRelease(cec_logical_address iDestination, bool bWait /* = true */)
 {
-  return m_busDevices[iDestination]->TransmitKeyRelease();
+  return m_busDevices[iDestination]->TransmitKeyRelease(bWait);
+}
+
+const char *CCECProcessor::ToString(const cec_device_type type)
+{
+  switch (type)
+  {
+  case CEC_DEVICE_TYPE_AUDIO_SYSTEM:
+    return "audio system";
+  case CEC_DEVICE_TYPE_PLAYBACK_DEVICE:
+    return "playback device";
+  case CEC_DEVICE_TYPE_RECORDING_DEVICE:
+      return "recording device";
+  case CEC_DEVICE_TYPE_RESERVED:
+      return "reserved";
+  case CEC_DEVICE_TYPE_TUNER:
+      return "tuner";
+  case CEC_DEVICE_TYPE_TV:
+      return "TV";
+  default:
+    return "unknown";
+  }
 }
 
 const char *CCECProcessor::ToString(const cec_menu_state state)
